function DriveControls(svg, project) {
	this.dispatch = d3.dispatch("change");
	
	this.svg = svg;
	
	this.project = project;
		
	
}

DriveControls.prototype = {
	updateTimers: [],
	controlsDir: "/images/controls/",
	
	drawControls: function() {
		var dt = this;
		
		//cancel all current timers
		for (var t=0; t<this.updateTimers.length; t++) {
			clearInterval(this.updateTimers[t]);
		}
		this.updateTimers = [];
		
		//clear all current controls
		this.svg.selectAll("use").remove();
		this.svg.selectAll(".controlbox-component").remove();
		this.svg.selectAll(".controlbox-label").remove();
		
		
		var validConfig = DriveConfiguration.validConfigs[this.project['drivemode']];
				
		
		for (var c=0; c<this.project.controlBox.length; c++) {
			if (this.project.controlBox[c]!=null) {
				var component = this.project.controlBox[c];
				var componentType = component.component;
				var componentInfo = this.components[componentType];
				
				
				var container = this.svg.select('#control' + (c+1));
				
				
				
				uc = this.svg.select("#layer1").append("g").attr("id", "usecomponent" + c).classed("controlbox-component", true);
				uc.attr("transform", "translate(" + container.attr("x") + ", " + container.attr("y") + ")");
				
				this.svg.select("#layer1").append("text")
					.attr("x", parseFloat(container.attr("x")) + parseFloat(container.attr("width"))/2 )
					.attr("y", parseFloat(container.attr("y")) + parseFloat(container.attr("height")) -6)
					.attr("dy", ".35em")
					.attr("text-anchor", "middle")
					.classed("controlbox-label", true)
					.text(component.name);
				
				/*uc.attr("x", container.attr("x"));
				uc.attr("y", container.attr("y"));
				*/
				
				switch (componentType) {
					case "switch": this.loadSwitch(c, uc, container);
						break;
					case "potentiometer": this.loadPotentiometer(c, uc, container);
						break;	
					case "pushbutton": this.loadPushbutton(c, uc, container);
						break;	
					case "analogmeter": this.loadAnalogmeter(c, uc, container);
						break;		
					case "currentsource": this.loadCurrentsource(c, uc, container);
						break;				
				}
					
				
				
	
			
			} else {
				
				
			}
		}
	
	},
	
	loadSwitch: function(c, container, placeholder) {
		var dc = this;
		
		d3.xml(this.controlsDir + "switch.svg").then(function(xml) {
			//var importedNode = document.importNode(xml.documentElement, true);
			var gElement = d3.select(xml).select('g'); 
			var nd = gElement.node();
			
			container.node().appendChild(gElement.node());
			
			container.select("#off").attr("opacity", 1);
			container.select("#on").attr("opacity", 0);
			dc.project.controlBox[c]['value'] = 0;
			dc.dispatch.call("change", dc, c);
			
			container.on("click", function() {
				d3.event.preventDefault();
				
				dc.project.controlBox[c]['value'] = dc.project.controlBox[c]['value']==0 ? 1 : 0;
				
				dc.dispatch.call("change", dc, c);
				
				container.select("#off").attr("opacity",  dc.project.controlBox[c]['value']==0 ? 1 : 0);
				container.select("#on").attr("opacity", dc.project.controlBox[c]['value']==0 ? 0 : 1);
			});
			
		});
	},
	loadPushbutton: function(c, container, placeholder) {
		var dc = this;
		
		d3.xml(this.controlsDir + "pushbutton.svg").then(function(xml) {
			//var importedNode = document.importNode(xml.documentElement, true);
			var gElement = d3.select(xml).select('g'); 
			var nd = gElement.node();
			
			container.node().appendChild(gElement.node());
			
			container.select("#depressed").attr("opacity",1);
			dc.project.controlBox[c]['value'] = 0;
			dc.dispatch.call("change", dc, c);
			
			container.on("mousedown", function() {
				d3.event.preventDefault();
				
				dc.project.controlBox[c]['value'] = 1;
				
				dc.dispatch.call("change", dc, c);
				
				container.select("#depressed").attr("opacity", 0);
			});
			
			container.on("mouseup", function() {
				d3.event.preventDefault();
				
				dc.project.controlBox[c]['value'] = 0;
				
				dc.dispatch.call("change", dc, c);
				
				container.select("#depressed").attr("opacity", 1);
			});
			
		});
	},
	
	loadPotentiometer: function(c, container, placeholder) {
		var dc = this;
		
		d3.xml(this.controlsDir + "potentiometer.svg").then(function(xml) {
			//var importedNode = document.importNode(xml.documentElement, true);
			var gElement = d3.select(xml).select('g'); 
			var nd = gElement.node();
			
			container.node().appendChild(gElement.node());
			
			dc.project.controlBox[c]['value']=0;
			dc.dispatch.call("change", dc, c);
			container.select("#knob").attr("transform", "rotate(40, 62.5, 62.5)");
			
			var md = false;
			
			var p1 = [parseFloat(placeholder.attr("x")) + parseFloat(placeholder.attr("width"))/2, parseFloat(placeholder.attr("y")) + parseFloat(placeholder.attr("height"))/2];

			container.on("mousedown", function() {
				d3.event.preventDefault();
				
				md = true;
				
			});
			
			dc.svg.on("mouseup." + placeholder.attr("id"), function() {
				md=false;
			});
			
			dc.svg.on("mousemove." + placeholder.attr("id"), function() {
				if (md) {
					var p2 = d3.mouse(dc.svg.select("#layer1").node());
					
					var a = Math.atan2(p2[1] - p1[1], p2[0] - p1[0]) * 180 / Math.PI;
					
					//make +ve
					if (a < 0) a = 360+a;
					
					//set origin at bottom
					a = (a + 270) % 360;
					
					
					if (a<40) a = 40;
					if (a>320) a = 320;
					
					container.select("#knob").attr("transform", "rotate(" + a + ", 62.5, 62.5)");
					
					//scale to float between 0 and 1
					dc.project.controlBox[c]['value'] = (a - 40)/280; 
					
					dc.dispatch.call("change", dc, c);
				}
			});
			
		});
	},
	
	loadCurrentsource: function(c, container, placeholder) {
		var dc = this;
		
		d3.xml(this.controlsDir + "currentsource.svg").then(function(xml) {
			//var importedNode = document.importNode(xml.documentElement, true);
			var gElement = d3.select(xml).select('g'); 
			var nd = gElement.node();
			
			container.node().appendChild(gElement.node());
			
			container.select("#updepressed").attr("opacity", 0);
			container.select("#downdepressed").attr("opacity", 0);
			dc.project.controlBox[c]['value'] = 4;
			dc.dispatch.call("change", dc, c);
			container.select("#displayValue").text(dc.project.controlBox[c]['value']);
				
			var up = container.select("#up");
			var down = container.select("#down");
			
			up.on("mousedown", function() {
				d3.event.preventDefault();
				container.select("#updepressed").attr("opacity", 1);
			});
			
			up.on("mouseup", function() {
				d3.event.preventDefault();
				
				if (dc.project.controlBox[c]['value']<20) {
					dc.project.controlBox[c]['value'] = dc.project.controlBox[c]['value']+1;
				}					
				
				container.select("#displayValue").text(dc.project.controlBox[c]['value']);
				
				dc.dispatch.call("change", dc, c);
				container.select("#updepressed").attr("opacity", 0);
			});
			
			down.on("mousedown", function() {
				d3.event.preventDefault();
				container.select("#downdepressed").attr("opacity", 1);
			});
			
			down.on("mouseup", function() {
				d3.event.preventDefault();
				
				if (dc.project.controlBox[c]['value']>4) {
					dc.project.controlBox[c]['value'] = dc.project.controlBox[c]['value']-1;
				}					
				
				container.select("#displayValue").text(dc.project.controlBox[c]['value']);
				
				dc.dispatch.call("change", dc, c);
				container.select("#downdepressed").attr("opacity", 0);
			});
			
		});
	},
	loadAnalogmeter: function(c, container, placeholder) {
		var dc = this;
		
		d3.xml(this.controlsDir + "analogmeter.svg").then(function(xml) {
			
			var gElement = d3.select(xml).select('g'); 
			var nd = gElement.node();
			
			container.node().appendChild(gElement.node());
			
			if (typeof(dc.project.controlBox[c]['value'])=="undefined") dc.project.controlBox[c]['value']=0;
			
			var updateFunc = function() {
				dc.dispatch.call("change", dc, c);
				
				var value = dc.project.controlBox[c]['value'];
				
				var startAngle = -44;
				var endAngle = 18;
				
				var angle = startAngle + (endAngle-startAngle) * value;
				
				container.select("#needle").attr("transform", "rotate(" + angle + ", 62.5, 107.5)");
			}
			
			updateFunc();
			var timer = setInterval(updateFunc, 100);
			
			dc.updateTimers.push(timer)
			
			
		});
	},
	
	components: {
			"led": {
				terminal1: "DOUT",
				terminal2: "",
				terminal3: "GND"
			}, 
			"switch": {
				terminal1: "24V",
				terminal2: "",
				terminal3: "DIN"
			}, 
			"potentiometer": {
				terminal1: "10V",
				terminal2: "AIN",
				terminal3: "GND"
			},
			"voltmeter": {
				terminal1: "AOUT",
				terminal2: "",
				terminal3: "GND"
			},
			"ammeter": {
				terminal1: "AOUT",
				terminal2: "",
				terminal3: "GND"
			}
		},
		
	driveTerminals: [
			{ name: "terminal1", type: "GND", color: "black" },
			{ name: "terminal2", type: "ANALOGIN", color: "green" },
			{ name: "terminal4", type: "10V", color: "red" },
			{ name: "terminal5", type: "ANALOGIN", color: "green" },
			{ name: "terminal7", type: "ANALOGOUT", color: "darkgreen" },
			{ name: "terminal9", type: "24V", color: "darkred" },
			{ name: "terminal10", type: "DIO", color: "blue" },
			{ name: "terminal11", type: "DIN", color: "darkblue" },
			{ name: "terminal12", type: "DIN", color: "darkblue" },
			{ name: "terminal13", type: "DIN", color: "darkblue" },
			{ name: "terminal14", type: "DIN", color: "darkblue" }
			
		],
		
}
