function DemoTerminals(svg, project) {
	this.dispatch = d3.dispatch("change");
	
	this.svg = svg;
	
	this.project = project;
	this.controlBox = project.controlBox;
	
	this.svg.select("#layer1").on("mousemove", this.mousemoveHandler.bind(this));
	this.svg.select("#layer1").on("click", this.clickHandler.bind(this));
	
	var dt = this;
	
	
	
	
	//set up component terminal
	for (var t=1; t<=2;  t++) {
		var svgt = this.svg.select("#terminal" + t);
		svgt.on("click", function() {
			d3.event.stopPropagation();
			if (dt.drawingTerminal==false) {				
				dt.drawingTerminal = d3.select(this).attr("id");
				
				dt.svg.select('#terminal1').style('fill', dt.terminalHighlight);
				dt.svg.select('#terminal2').style('fill', dt.terminalHighlight);				
				
			} else {
				var terminal1 = dt.drawingTerminal;
				var terminal2 = d3.select(this).attr("id");
				
				var valid = terminal1!=terminal2;		//not joining to same terminal
				
				if (valid) {
					dt.connected = true;
					
					dt.clickHandler();
					
					dt.dispatch.call("change");
					
					dt.drawTerminals();
					
				}
			}
		});
	}

	
}


DemoTerminals.prototype = {
	terminalColor: "#999999",
	terminalHighlight: "#EA5A00",
	
	connected: false,
		
	checkComplete: function() {
		
		
		return true;
	},	
	

	drawTerminals: function() {
		var dt = this;
		
		this.svg.selectAll(".componentLine").remove();
		
	
		dt.svg.select('#terminal1').style('fill', dt.terminalColor);
		dt.svg.select('#terminal2').style('fill', dt.terminalColor);				
		
		
		
		if (this.connected) {
				
				
				var terminal1 = this.svg.select("#terminal1");
				var terminal2 = this.svg.select("#terminal2");
				
				var xy1 = [terminal1.attr("cx"), terminal1.attr("cy")];
				var xy2 = [terminal2.attr("cx"), terminal2.attr("cy")];
					 	
		   	 	var path = this.svg.select('#layer1')
		             .append('path')
		             .style('stroke', "#ee7930")
		             .style('stroke-width', '2')
		             .style('stroke-linecap', 'round')
		             .style('stroke-linejoin', 'round')
		             .style('fill', 'none')
		             .attr('class', "componentLine")
		             .attr('d', this.lineFunction(xy1, xy2));

		}
	
	},
	
	drawingTerminal: false,
	drawingPath: null,
	mousemoveHandler: function() {
		if (this.drawingTerminal!==false) {
			var svgt = this.svg.select("#" + this.drawingTerminal);
			
			var xy0 = [svgt.attr("cx"), svgt.attr("cy")];
			
			var xy1 = d3.mouse(this.svg.select("#layer1").node()).map(function(x){ return x - 1; });
			
			var line = this.lineFunction(xy0, xy1)
			
			if (this.drawingPath==null) {
				//create the path
				
				this.drawingPath = this.svg.select('#layer1')
                     .append('path')
                     .style('stroke', this.terminalHighlight)
                     .style('stroke-width', '1px')
                     .style('fill', 'none')
                     .attr('d', line);
			} else {
				//move the path
				this.drawingPath.attr("d", line);
			}
		
		}
	
	},
	
	clickHandler: function() {
		if (this.drawingTerminal!==false) {
			this.drawingTerminal = false;
			this.drawingPath.remove();
			this.drawingPath = null;
			
			this.svg.select('#terminal1').style('fill', this.terminalColor);
			this.svg.select('#terminal2').style('fill', this.terminalColor);				
		
		
		}
	},
	
	
		
	lineFunction: function(start, end) {
		
		
		var midx = (parseFloat(start[0]) + parseFloat(end[0])) / 2;
		
		var xy1 = [midx, start[1]];
		var xy2 = [midx, end[1]];
		
		return d3.line()
			.x(function(d){ return d[0]; })
			.y(function(d){ return d[1]; })([start, xy1, xy2, end]);
	
		
		
	}
}
