function Drive(svg, project, parameterInfo) {
	this.dispatch = d3.dispatch("change", "view");
	var that = this;
	
	this.svg = svg;
	
	this.project = project;
	this.parameterInfo = parameterInfo;
	this.parameterValues = {};
		
	this.setupMenus();
	this.setupDisplay();
	
	
	this.dispatch.on("change.internal", function(param) {
		that.changeParameter(param);
	});
	
	this.btnEscape = svg.select("path#btn1");
	this.btnDown = svg.select("path#btn2");
	this.btnStart = svg.select("path#btn3");
	this.btnStop = svg.select("path#btn4");
	this.btnUp = svg.select("path#btn5");
	this.btnEnter = svg.select("path#btn6");

	
	this.btnEscape.on("click", this.btnEscape_click.bind(this)).style("cursor", "pointer");
	this.btnDown.on("mouseup", this.btnDown_up.bind(this)).style("cursor", "pointer");
	this.btnDown.on("mousedown", this.btnDown_down.bind(this));
	this.btnStart.on("click", this.btnStart_click.bind(this)).style("cursor", "pointer");
	this.btnStop.on("click", this.btnStop_click.bind(this)).style("cursor", "pointer");
	this.btnUp.on("mouseup", this.btnUp_up.bind(this)).style("cursor", "pointer");
	this.btnUp.on("mousedown", this.btnUp_down.bind(this));	
	this.btnEnter.on("click", this.btnEnter_click.bind(this)).style("cursor", "pointer");
	
	this.svg.on("mouseup", this.mouseUp.bind(this));
	
	this.uiState = "status";
	this.currentMenu = 0;
	
	this.updateDisplay();
	
	
	setInterval(function() {
		that.tick();
	}, this.tickInterval);
}	

Drive.prototype = {
	started: false,					/* set to true once all menus, etc have been loaded */
	tickInterval: 100,
	parameterViewParameter: false,	/* true if currently displaying parameter name instead of value	*/
	parameterViewInterval: null,	/* timer to reset above property */
	currentParamaterValue: null,	/* the value of the parameter currently being edited */
	buttonHeldRepeat: 0,			/* number of times function repeated */
	buttonHeldInterval: null,
	
	
	changeParameter: function(param) {
		if (param=="11.034") {
			//handle mode change
			switch (this.getParameterValue("11.034").toString()) {
				case "0": 
					//AV
					this.setParameterValue("01.014", "0");
					this.setParameterValue("07.007", "6");
					this.setParameterValue("07.011", "6");
					this.setParameterValue("07.051", "0");
					this.setParameterValue("07.010", "1.036");
					this.setParameterValue("07.014", "1.037");
					this.setParameterValue("08.125", "1.041");
					this.setParameterValue("08.085", "0");
					this.setParameterValue("09.025", "0");
					this.setParameterValue("14.003", "0");
					this.setParameterValue("14.004", "0");
					this.setParameterValue("14.016", "0");
					
					break;
				case "1": 
					//AI
					this.setParameterValue("01.014", "0");
					this.setParameterValue("07.007", "4");
					this.setParameterValue("07.011", "6");
					this.setParameterValue("07.051", "0");
					this.setParameterValue("07.010", "1.036");
					this.setParameterValue("07.014", "1.037");
					this.setParameterValue("08.125", "1.041");
					this.setParameterValue("08.085", "0");
					this.setParameterValue("09.025", "0");
					this.setParameterValue("14.003", "0");
					this.setParameterValue("14.004", "0");
					this.setParameterValue("14.016", "0");
					
					break;
				case "2": 
					//AV.Pr
					this.setParameterValue("01.014", "1");
					this.setParameterValue("07.007", "6");
					this.setParameterValue("07.011", "7");
					this.setParameterValue("07.051", "0");
					this.setParameterValue("07.010", "1.036");
					this.setParameterValue("07.014", "1.046");
					this.setParameterValue("08.125", "1.045");
					this.setParameterValue("08.085", "0");
					this.setParameterValue("09.025", "0");
					this.setParameterValue("14.003", "0");
					this.setParameterValue("14.004", "0");
					this.setParameterValue("14.016", "0");
					
					break;
			
				case "3": 
					//AI.Pr
					this.setParameterValue("01.014", "1");
					this.setParameterValue("07.007", "4");
					this.setParameterValue("07.011", "7");
					this.setParameterValue("07.051", "0");
					this.setParameterValue("07.010", "1.036");
					this.setParameterValue("07.014", "1.046");
					this.setParameterValue("08.125", "1.045");
					this.setParameterValue("08.085", "0");
					this.setParameterValue("09.025", "0");
					this.setParameterValue("14.003", "0");
					this.setParameterValue("14.004", "0");
					this.setParameterValue("14.016", "0");
					
					break;
				case "4": 
					//Preset
					this.setParameterValue("01.014", "3");
					this.setParameterValue("07.007", "6");
					this.setParameterValue("07.011", "7");
					this.setParameterValue("07.051", "0");
					this.setParameterValue("07.010", "1.036");
					this.setParameterValue("07.014", "1.046");
					this.setParameterValue("08.125", "1.045");
					this.setParameterValue("08.085", "0");
					this.setParameterValue("09.025", "0");
					this.setParameterValue("14.003", "0");
					this.setParameterValue("14.004", "0");
					this.setParameterValue("14.016", "0");
					
					break;
				case "5": 
					//Pad
					this.setParameterValue("01.014", "4");
					this.setParameterValue("07.007", "6");
					this.setParameterValue("07.011", "6");
					this.setParameterValue("07.051", "0");
					this.setParameterValue("07.010", "1.036");
					this.setParameterValue("07.014", "1.037");
					this.setParameterValue("08.125", "1.041");
					this.setParameterValue("08.085", "0");
					this.setParameterValue("09.025", "0");
					this.setParameterValue("14.003", "0");
					this.setParameterValue("14.004", "0");
					this.setParameterValue("14.016", "0");
					
					break;	
				case "6": 
					//Pad.ref
					this.setParameterValue("01.014", "6");
					this.setParameterValue("07.007", "6");
					this.setParameterValue("07.011", "6");
					this.setParameterValue("07.051", "0");
					this.setParameterValue("07.010", "1.036");
					this.setParameterValue("07.014", "1.037");
					this.setParameterValue("08.125", "1.041");
					this.setParameterValue("08.085", "0");
					this.setParameterValue("09.025", "0");
					this.setParameterValue("14.003", "0");
					this.setParameterValue("14.004", "0");
					this.setParameterValue("14.016", "0");
					
					break;
				case "7": 
					//E.Pot
					this.setParameterValue("01.014", "3");
					this.setParameterValue("07.007", "6");
					this.setParameterValue("07.011", "7");
					this.setParameterValue("07.051", "0");
					this.setParameterValue("07.010", "1.036");
					this.setParameterValue("07.014", "9.027");
					this.setParameterValue("08.125", "9.026");
					this.setParameterValue("08.085", "0");
					this.setParameterValue("09.025", "1.021");
					this.setParameterValue("14.003", "0");
					this.setParameterValue("14.004", "0");
					this.setParameterValue("14.016", "0");
					
					break;	
				case "8": 
					//Torque
					this.setParameterValue("01.014", "0");
					this.setParameterValue("07.007", "4");
					this.setParameterValue("07.011", "6");
					this.setParameterValue("07.051", "0");
					this.setParameterValue("07.010", "1.036");
					this.setParameterValue("07.014", "4.008");
					this.setParameterValue("08.125", "4.011");
					this.setParameterValue("08.085", "0");
					this.setParameterValue("09.025", "0");
					this.setParameterValue("14.003", "0");
					this.setParameterValue("14.004", "0");
					this.setParameterValue("14.016", "0");
					
					break;			
				case "9": 
					//PID
					this.setParameterValue("01.014", "1");
					this.setParameterValue("07.007", "4");
					this.setParameterValue("07.011", "6");
					this.setParameterValue("07.051", "0");
					this.setParameterValue("07.010", "0.000");
					this.setParameterValue("07.014", "0.000");
					this.setParameterValue("08.125", "14.008");
					this.setParameterValue("08.085", "0");
					this.setParameterValue("09.025", "0");
					this.setParameterValue("14.003", "7.021");
					this.setParameterValue("14.004", "7.001");
					this.setParameterValue("14.016", "1.036");
					
					break;	
			}
			
			this.dispatch.call("change", this, "1.014");		//make sure the save callback gets run again
		
		}
		
		if (param=="6.004") {
			switch (parseInt(this.getParameterValue("6.004"))) {
				case 0:
				case 2:
				case 5:
					this.setParameterValue("6.040", 0);		//non-latching
					break;
				default:
					this.setParameterValue("6.040", 1);
					break;
			
			}
		
		
		}
		
	},
	
	mouseUp: function() {
		if (this.buttonHeldInterval) {
			this.buttonHeldRepeat = 0;
			clearInterval(this.buttonHeldInterval);
			this.buttonHeldInterval = null;
		}
	},
	
	btnEscape_click: function() {
		switch (this.uiState) {
			case "status":
				
				break;
			case "menuSelect":
				this.uiState = "status";
			
				break;
			case "menu0Select":
				this.uiState = "status";
			
				break;
					
			case "parameterSelect":
				this.uiState = "menuSelect";
			
				break;	
				
			case "parameterView":
				switch (parseInt(this.getParameterValue("11.044"))) {
					case 0:
					case 1:
					case 3:
					case 4:
						this.uiState = "menu0Select";
						break;
					case 2:
						this.uiState = "parameterSelect";
						break;
				}
				
				break;
				
			case "parameterEdit":
			
				this.uiState = "parameterView";
				break;		
		}
		
		this.updateDisplay();
	},
	btnDown_down: function() {
		var that = this;
		this.buttonHeldInterval = setInterval(function() {
			that.btnDown_click();
			that.buttonHeldRepeat += 1;			
		}, 200);
	},
	btnDown_up: function() {
		if (this.buttonHeldRepeat==0) this.btnDown_click();		//only register a click if we've not been repeating
	},
	btnDown_click: function() {
		
		switch (this.uiState) {
			case "status":
				if (this.getParameterValue("11.034")==5 || this.getParameterValue("11.034")==6) {
					//pad / pad.ref mode
					var d = 0;
					if (this.buttonHeldRepeat>0) {
						var d = Math.floor(this.buttonHeldRepeat/10);		//increment digit every 10 repeats when button held down
						if (Math.floor(this.buttonHeldRepeat/10)==this.buttonHeldRepeat/10) this.buttonHeldRepeat++;	//add 1 to sort out the digit rollover
					}

					var inc = Math.pow(10,d-2);							
					if (inc>10) inc = 10;	//limit the increment
						
					curVal = parseFloat(this.getParameterValue("1.017"));
					curVal -= inc;
					
					if (parseInt(this.getParameterValue("1.010"))==1) {			//bipolar ref enabled
						minVal = -parseFloat(this.getParameterValue("1.006"));
					} else {					
						minVal = parseFloat(this.getParameterValue("1.007"));
					}
					if (curVal<minVal) curVal = minVal;
					
					this.setParameterValue("1.017", curVal.toFixed(2));
				} else {
					switch (parseInt(this.getParameterValue("11.044"))) {
						case 0:
						case 1:
						case 3:
						case 4:
							this.uiState = "menu0Select";
							break
						case 2:
							this.uiState = "menuSelect";
							break;
					}
				}
			case "menuSelect":
				this.currentMenu --;
				if (this.currentMenu<0) this.currentMenu=this.menus.length-1;
				
				
				break;
			case "menu0Select":
				var currMenu = this.menus[0];
				
				currMenu.currentParameter--
				
				if (parseInt(this.getParameterValue("11.044"))==0 || parseInt(this.getParameterValue("11.044"))==3 || parseInt(this.getParameterValue("11.044"))==4) {
					//restrict to first 10 parameters
					if (currMenu.currentParameter<0) currMenu.currentParameter = 9;
				} else {
					if (currMenu.currentParameter<0) currMenu.currentParameter = currMenu.parameters.length-1;
				}
				break;
				
				
			case "parameterSelect":
				var currMenu = this.menus[this.currentMenu];
				
				currMenu.currentParameter--
				
				if (parseInt(this.getParameterValue("11.044"))==0 || parseInt(this.getParameterValue("11.044"))==3 || parseInt(this.getParameterValue("11.044"))==4) {
					//restrict to first 10 parameters
					if (currMenu.currentParameter<0) currMenu.currentParameter = 9;
				} else {
					if (currMenu.currentParameter<0) currMenu.currentParameter = currMenu.parameters.length-1;
				}
				
				break;
				
			case "parameterView":
				var currMenu = this.menus[this.currentMenu];
				
				currMenu.currentParameter--
				
				if (parseInt(this.getParameterValue("11.044"))==0 || parseInt(this.getParameterValue("11.044"))==3 || parseInt(this.getParameterValue("11.044"))==4) {
					//restrict to first 10 parameters
					if (currMenu.currentParameter<0) currMenu.currentParameter = 9;
				} else {
					if (currMenu.currentParameter<0) currMenu.currentParameter = currMenu.parameters.length-1;
				}
				
				this.parameterViewParameter = true;
				if (this.parameterViewInterval!=null) clearTimeout(this.parameterViewInterval);
				var that = this;
				this.parameterViewInterval = setTimeout(function() {
					that.parameterViewParameter = false;
					that.updateDisplay();
				}, 1000);
				
				
				break;	
				
			case "parameterEdit":
				var currMenu = this.menus[this.currentMenu];
				var currParameter = currMenu.parameters[currMenu.currentParameter];
				
				switch (currParameter.type) {
					case "Num":
						this.editNum(-1);
						
						break;
					case "Txt":
					case "Bit":
						var value = parseInt(this.currentParameterValue);
						if (typeof(currParameter["options"][value - 1])!="undefined") {
							this.currentParameterValue = value-1;
						}
												
						break;	
					default:
						
				}
			
				break;				
			
		}
		
		this.updateDisplay();
	},
	btnStart_click: function() {
		if (this.getParameterValue("11.034")==5) {
			//pad mode
			//set fwd/reverse to 1 depending on value of Pr 17 (1.010)
			if (this.getParameterValue("1.010")==0) {
				this.setParameterValue("6.030", 1);
				this.setParameterValue("6.032", 0);
			} else {
				this.setParameterValue("6.030", 0);
				this.setParameterValue("6.032", 1);
			}
		}
	},
	btnStop_click: function() {
		if (this.getParameterValue("11.034")==5) {
			//pad mode
			//set forward/reverse to 0;
			this.setParameterValue("6.030", 0);
			this.setParameterValue("6.032", 0);
		}
	},
	
	btnUp_down: function() {
		var that = this;
		this.buttonHeldInterval = setInterval(function() {
			that.btnUp_click();			
			that.buttonHeldRepeat += 1;			
		}, 200);
	},
	btnUp_up: function() {
		if (this.buttonHeldRepeat==0) this.btnUp_click();		//only register a click if we've not been repeating
	},
	btnUp_click: function() {
		switch (this.uiState) {
			case "status":
				
				if (this.getParameterValue("11.034")==5 || this.getParameterValue("11.034")==6) {
					//pad mode // pad.ref mode
					var d = 0;
					if (this.buttonHeldRepeat>0) {
						var d = Math.floor(this.buttonHeldRepeat/10);		//increment digit every 10 repeats when button held down
						if (Math.floor(this.buttonHeldRepeat/10)==this.buttonHeldRepeat/10) this.buttonHeldRepeat++;	//add 1 to sort out the digit rollover
					}

					var inc = Math.pow(10,d-2);							
					if (inc>10) inc = 10;	//limit the increment
					
					curVal = parseFloat(this.getParameterValue("1.017"));
					curVal += inc;
					
					maxVal = parseFloat(this.getParameterValue("1.006"));
					if (curVal>maxVal) curVal = maxVal;
					
					this.setParameterValue("1.017", curVal.toFixed(2));
				} else {
					switch (parseInt(this.getParameterValue("11.044"))) {
						case 0:
						case 1:
						case 3:
						case 4:
							this.uiState = "menu0Select";
							break
						case 2:
							this.uiState = "menuSelect";
							break;
					}
				}
				break;
				
			case "menuSelect":
				
				this.currentMenu ++;
				if (this.currentMenu>this.menus.length-1) this.currentMenu=0;
				
				break;
			
			case "menu0Select":
				
				var currMenu = this.menus[0];
				
				currMenu.currentParameter++
				
				if (parseInt(this.getParameterValue("11.044"))==0 || parseInt(this.getParameterValue("11.044"))==3 || parseInt(this.getParameterValue("11.044"))==4) {
					//restrict to first 10 parameters
					if (currMenu.currentParameter>9) currMenu.currentParameter = 0;
				} else {
					if (currMenu.currentParameter>currMenu.parameters.length-1) currMenu.currentParameter = 0;
				}
				
				
				break;	
				
			case "parameterSelect":

				var currMenu = this.menus[this.currentMenu];
				
				currMenu.currentParameter++
				
				if (parseInt(this.getParameterValue("11.044"))==0 || parseInt(this.getParameterValue("11.044"))==3 || parseInt(this.getParameterValue("11.044"))==4) {
					//restrict to first 10 parameters
					if (currMenu.currentParameter>9) currMenu.currentParameter = 0;
				} else {
					if (currMenu.currentParameter>currMenu.parameters.length-1) currMenu.currentParameter = 0;
				}
				
				break;	
				
			case "parameterView":
				var currMenu = this.menus[this.currentMenu];
				
				currMenu.currentParameter++
				
				if (parseInt(this.getParameterValue("11.044"))==0 || parseInt(this.getParameterValue("11.044"))==3 || parseInt(this.getParameterValue("11.044"))==4) {
					//restrict to first 10 parameters
					if (currMenu.currentParameter>9) currMenu.currentParameter = 0;
				} else {
					if (currMenu.currentParameter>currMenu.parameters.length-1) currMenu.currentParameter = 0;
				}
				
				this.parameterViewParameter = true;
				if (this.parameterViewInterval!=null) clearTimeout(this.parameterViewInterval);
				var that = this;
				this.parameterViewInterval = setTimeout(function() {
					that.parameterViewParameter = false;
					that.updateDisplay();
				}, 1000);
				
				
				break;	
			
			case "parameterEdit":
				var currMenu = this.menus[this.currentMenu];
				var currParameter = currMenu.parameters[currMenu.currentParameter];
				
				switch (currParameter.type) {
					case "Num":
						this.editNum(1);
						
						break;
					case "Txt":
					case "Bit":
						var value = parseInt(this.currentParameterValue);
						if (typeof(currParameter["options"][value + 1])!="undefined") {
							this.currentParameterValue = value+1;
						}
						break;
					
					default:
						
				}
			
				break;	
		}
		
		this.updateDisplay();
	},
	btnEnter_click: function() {
		switch (this.uiState) {
			case "status":
				
				switch (parseInt(this.getParameterValue("11.044"))) {
					case 0:
					case 1:
					case 3:
					case 4:
						this.uiState = "menu0Select";
						break
					case 2:
						this.uiState = "menuSelect";
						break;
				}
				
				break;
			case "menuSelect":
				
				this.uiState = "parameterSelect";
				break;
			case "menu0Select":
				
				this.uiState = "parameterView";
				break;
					
			case "parameterSelect":
				
				this.uiState = "parameterView";
				break;
				
			case "parameterView":
				//check if read only parameter
				var currMenu = this.menus[this.currentMenu];
				var currParameter = currMenu.parameters[currMenu.currentParameter];
				
				if (this.parameterViewParameter) {
					//don't edit, just switch back to the parameter value on the display and clear the interval
					if (this.parameterViewInterval!=null) clearTimeout(this.parameterViewInterval);
					this.parameterViewParameter = false;
				} else {
					if (currParameter.readonly) {
						for (var d=1; d<=this.numDigits; d++) {
							this.selectDigit(d).classed("roblink", true);
						}
						
						var that = this;
						setTimeout(function() {
							for (var d=1; d<=that.numDigits; d++) {
								that.selectDigit(d).classed("roblink", false);
							}
						}, 800);
					} else {
				
						this.currentParameterValue = this.getParameterValue(currParameter.storedParameterName);
						this.uiState = "parameterEdit";
					}
				}
				break;
				
			case "parameterEdit":
				//save the parameter back
				var currMenu = this.menus[this.currentMenu];
				var currParameter = currMenu.parameters[currMenu.currentParameter];
				
				this.setParameterValue(currParameter.storedParameterName, this.currentParameterValue);
				
				this.dispatch.call("change", this, currParameter.storedParameterName);
				
				this.uiState = "parameterView";
				break;				
		
		}
		
		this.updateDisplay();
	},
	
	updateDisplay: function() {
		d3.select('#not-implemented').style('display', 'none');
		
		if (parseInt(this.getParameterValue("11.031"))==2) {
			this.showNotImplemented("RFC-A Mode not implemented in this demo.<br>To remove this message please use the keypad to return to OpenLP mode");
		}

		for (var d=1; d<=this.numDigits; d++) {
			this.selectDigit(d).classed("blink", false);
		}
		this.svg.select("#ledKeypad").attr("fill-opacity", 0);
		this.svg.select("#ledFwd").attr("fill-opacity", 0);
		this.svg.select("#ledRev").attr("fill-opacity", 0);
		this.svg.select("#ledV").attr("fill-opacity", 0);
		this.svg.select("#ledA").attr("fill-opacity", 0);
		this.svg.select("#ledHz").attr("fill-opacity", 0);
		this.svg.select("#ledRpm").attr("fill-opacity", 0);
		this.svg.select("#ledPc").attr("fill-opacity", 0);
		
		if (this.getParameterValue("11.034")==5) {
			this.svg.select("#ledKeypad").attr("fill-opacity", 1);
		}
		if (parseFloat(this.getParameterValue("5.001"))>0) {
			this.svg.select("#ledFwd").attr("fill-opacity", 1);
		}
		if (parseFloat(this.getParameterValue("5.001"))<0) {
			this.svg.select("#ledRev").attr("fill-opacity", 1);
		}
		
		switch (this.uiState) {
			case "status":
				
				this.display("OPeNLP");
				var value = this.getParameterValue("10.101");
				if (typeof(value)!="undefined") {
					if (parseFloat(this.getParameterValue("5.001"))!=0) {
						//display the frequency
						var parameterToDisplay = this.getParameterValue("11.018");
						var parameterToDisplayI = this.getParameterInfo(parameterToDisplay);
						
						switch (parameterToDisplayI['unit']) {
							case "V":
								this.svg.select("#ledV").attr("fill-opacity", 1);
								break;
							case "A":
								this.svg.select("#ledA").attr("fill-opacity", 1);
								break;
							case "Hz":
								this.svg.select("#ledHz").attr("fill-opacity", 1);
								break;
							case "rpm":
								this.svg.select("#ledRpm").attr("fill-opacity", 1);
								break;
							case "%":
								this.svg.select("#ledPc").attr("fill-opacity", 1);
								break;																												
						}
						//this.svg.select("#ledHz").attr("fill-opacity", 1);
						this.displayValue(parameterToDisplayI, parseFloat(this.getParameterValue(parameterToDisplay) * parseFloat(this.getParameterValue("11.021"))).toFixed(2));
					} else {
						this.displayValue(this.getParameterInfo("10.101"), value);
					}
				}
		
				break;
				
			case "menuSelect":
				this.selectDigit(2).classed("blink", true);
				this.selectDigit(3).classed("blink", true);
				
				var currMenu = this.menus[this.currentMenu];
				var currParameter = currMenu.parameters[currMenu.currentParameter];
				
				this.display(" " + currMenu.menuName + "." + currParameter.parameterName);
			
				break;
			case "menu0Select":
				this.selectDigit(5).classed("blink", true);
				this.selectDigit(6).classed("blink", true);		
				
				var currMenu = this.menus[0];
				var currParameter = currMenu.parameters[currMenu.currentParameter];		
				
				this.display("Pr.  " + currParameter.parameterName.slice(-2));
				
				break;
			case "parameterSelect": 
				this.selectDigit(4).classed("blink", true);
				this.selectDigit(5).classed("blink", true);
				this.selectDigit(6).classed("blink", true);				
				
				var currMenu = this.menus[this.currentMenu];
				var currParameter = currMenu.parameters[currMenu.currentParameter];
				
				this.display(" " + currMenu.menuName + "." + currParameter.parameterName);
				
				break;	
			case "parameterView":
				var currMenu = this.menus[this.currentMenu];
				var currParameter = currMenu.parameters[currMenu.currentParameter];
				
				if (this.parameterViewParameter) {
					//display the parameter name (temporarily)
					switch (parseInt(this.getParameterValue("11.044"))) {
						case 0:
						case 1:
						case 3:
						case 4:
							this.display("Pr.  " + currParameter.parameterName.slice(-2));
							break
						case 2:
							this.display(" " + currMenu.menuName + "." + currParameter.parameterName);
							break;
					}
					
				} else {
					
					var value = this.getParameterValue(currParameter.storedParameterName);
					this.displayValue(currParameter, value);
					
					this.dispatch.call("view", this, currParameter.storedParameterName);
				}
				
				if (typeof(currParameter['unit'])!="undefined") {
					switch (currParameter['unit']) {
						case "V":
							this.svg.select("#ledV").attr("fill-opacity", 1);
							break;
						case "A":
							this.svg.select("#ledA").attr("fill-opacity", 1);
							break;
						case "Hz":
							this.svg.select("#ledHz").attr("fill-opacity", 1);
							break;
						case "rpm":
							this.svg.select("#ledRpm").attr("fill-opacity", 1);
							break;
						case "%":
							this.svg.select("#ledPc").attr("fill-opacity", 1);
							break;																												
					}
				
				}
				
				break;
				
			case "parameterEdit":
				var currMenu = this.menus[this.currentMenu];
				var currParameter = currMenu.parameters[currMenu.currentParameter];
				
				switch (currParameter.type) {
					case "Num": 
						//flash last digit
						this.selectDigit(this.numDigits).classed("blink", true);
						break;
						
					case "Txt":
						for (var d=1; d<=this.numDigits; d++) {
							this.selectDigit(d).classed("blink", true);
						}
						break;
						
					case "Bit":
						for (var d=1; d<=this.numDigits; d++) {
							this.selectDigit(d).classed("blink", true);
						}
						break;
				
				}
				
				var value = this.getParameterValue(currParameter.storedParameterName);
				this.displayValue(currParameter, this.currentParameterValue);
				
				if (typeof(currParameter['unit'])!="undefined") {
					switch (currParameter['unit']) {
						case "V":
							this.svg.select("#ledV").attr("fill-opacity", 1);
							break;
						case "A":
							this.svg.select("#ledA").attr("fill-opacity", 1);
							break;
						case "Hz":
							this.svg.select("#ledHz").attr("fill-opacity", 1);
							break;
						case "rpm":
							this.svg.select("#ledRpm").attr("fill-opacity", 1);
							break;
						case "%":
							this.svg.select("#ledPc").attr("fill-opacity", 1);
							break;																												
					}
				
				}
				
				break;
		}
		
	
	},
	
	tick: function() {
			
		if (!this.started) {
			 return;
		}
		this.updateDisplay();
		
		var currentFreq = parseFloat(this.getParameterValue("5.001"));
		var currentDir = currentFreq < 0 ? -1 : 1;
		var referenceFreq = 0;
		var referenceDir = 0;
		var aTime = 0;
		
		if (this.getParameterValue("6.029")==0) {
			if (currentFreq==0) {
				//nothing to do
				this.setParameterValue("10.101", 0);
				return;	
			}
			
			referenceFreq = 0;
			
			switch (parseInt(this.getParameterValue("6.004"))) {
				case 1:
				case 3:
				case 4:
					//not-stop, obey deceleration
					referenceDir = currentDir;
					//remove fwd / rev
					this.setParameterValue("6.030", 0);
					this.setParameterValue("6.032", 0);
					
					break;
				default:
					break;
			}
			
		} else {
			this.setParameterValue("10.101", 1);	
			
			//update the e.pot if necessary
			if (this.getParameterValue("11.034")==7) {
				var potDir = 0;
				if (this.getParameterValue("9.026")==1) {
					potDir=1;
				} else if (this.getParameterValue("9.027")==1) {
					potDir=-1;
				}
				
				if (potDir!=0) {
					//9.023 contains the time to reach max frequency
					var potRate = parseFloat(this.getParameterValue("9.023"));
					var hzPerSecond = parseFloat(this.getParameterValue("1.006"))/potRate;
					var hzInc = (hzPerSecond / 1000) * this.tickInterval;
					
					var dest = this.getParameterValue("9.025");			//get where to store the parameter - default 1.021 (preset reference 1)
					var currentValue = parseFloat(this.getParameterValue(dest));
					currentValue += potDir * hzInc;
					
					//limit max / min freq
					if (currentValue>parseFloat(this.getParameterValue("1.006"))) {
						currentValue = parseFloat(this.getParameterValue("1.006"));
					} 
					if (currentValue<parseFloat(this.getParameterValue("1.007"))) {
						currentValue = parseFloat(this.getParameterValue("1.007"));
					} 
					
					this.setParameterValue(dest, currentValue);
				
				}
			}
			
			//get our target direction
			switch (parseInt(this.getParameterValue("6.004"))) {
				case 0:
				case 1:
				case 5: 
				case 6:
					//run forward, run reverse
					if (this.getParameterValue("6.030")==1 && this.getParameterValue("6.032")==0) {
						referenceDir = 1;	//going forward
					} else if (this.getParameterValue("6.030")==0 && this.getParameterValue("6.032")==1) { 
						referenceDir = -1;	//going reverse
					} else {
						//no direction set, set target frequency to 0;
						referenceFreq = 0;
						
						if (currentFreq==0) {
							//nothing to do
							this.setParameterValue("10.101", "1");
							return;
						}
					}
					break;
				case 2:
				case 3:
					//run, reverse
					if (this.getParameterValue("6.030")==1) {
						if (this.getParameterValue("6.032")==0) {
							referenceDir = 1;
						} else {
							referenceDir = -1;
						}
					
					} else {
						//not run
						referenceDir = 0;
						referenceFreq = 0;
					}
					break;
			}	
			
			if (referenceDir!=0) {
				//get our target frequency
				if (this.getParameterValue("1.014")==0) {
					//a1/a2
					if (this.getParameterValue("1.041")==0) {
						referenceFreq = parseFloat(this.getParameterValue("1.036"));
					} else {
						referenceFreq = parseFloat(this.getParameterValue("1.037"));
					}
				} else if (this.getParameterValue("1.014")==1) {
					//A1.pr
					if (this.getParameterValue("1.045")=="0" && this.getParameterValue("1.046")=="0") {
						referenceFreq = parseFloat(this.getParameterValue("1.036"));
					} else if (this.getParameterValue("1.045")=="0" && this.getParameterValue("1.046")=="1") {
						referenceFreq = parseFloat(this.getParameterValue("1.022"));
					} else if (this.getParameterValue("1.045")=="1" && this.getParameterValue("1.046")=="0") {
						referenceFreq = parseFloat(this.getParameterValue("1.023"));
					} else if (this.getParameterValue("1.045")=="1" && this.getParameterValue("1.046")=="1") {
						referenceFreq = parseFloat(this.getParameterValue("1.024"));
					}
				} else if (this.getParameterValue("1.014")==3) {
					//Preset
					if (this.getParameterValue("11.034")==7) {
						//obey the motorized pot destination (9.025) value
						referenceFreq = parseFloat(this.getParameterValue(this.getParameterValue("9.025")));
					} else {
						if (this.getParameterValue("1.045")=="0" && this.getParameterValue("1.046")=="0") {
							referenceFreq = parseFloat(this.getParameterValue("1.021"));
						} else if (this.getParameterValue("1.045")=="0" && this.getParameterValue("1.046")=="1") {
							referenceFreq = parseFloat(this.getParameterValue("1.022"));
						} else if (this.getParameterValue("1.045")=="1" && this.getParameterValue("1.046")=="0") {
							referenceFreq = parseFloat(this.getParameterValue("1.023"));
						} else if (this.getParameterValue("1.045")=="1" && this.getParameterValue("1.046")=="1") {
							referenceFreq = parseFloat(this.getParameterValue("1.024"));
						}
					}
				} else if (this.getParameterValue("1.014")==4) {
					//Pad
					referenceFreq = parseFloat(this.getParameterValue("1.017"));
				} else if (this.getParameterValue("1.014")==6) {
					//Pad.ref
					referenceFreq = parseFloat(this.getParameterValue("1.017"));
				} 
			} else {
				//no fwd/rev set so decellerate to 0
				referenceDir = currentDir;
				referenceFreq = 0;
			
			}
			
			
			
		}
			
		
		if (parseInt(this.getParameterValue("1.010"))==1) {		//bipolar ref not enabled
			if (referenceFreq<0) {
				referenceDir = -1;
				referenceFreq = -referenceFreq;
			} else {
				referenceDir = 1;
			}
		}
		
		//if we're going in the wrong direction, set reference freq to 0.
		if (referenceDir>0 && currentFreq<0) {
			referenceDir = currentDir;		//decelerate to 0 
			referenceFreq = 0;
		} else if (referenceDir<0 && currentFreq>0) {
			referenceDir = currentDir;		//decelerate to 0 
			referenceFreq = 0;
		}
		
		
		var absFreq = Math.abs(currentFreq);
		
		var acc;
		var ad;
		//work out if we're accelerating or decelerating
		if (referenceFreq<absFreq) {
			//decel
			this.setParameterValue("10.101", "6");
			acc = parseFloat(this.getParameterValue("2.021"));
			ad = -1;
		} else {
			//accel
			this.setParameterValue("10.101", "4");
			var acc = parseFloat(this.getParameterValue("2.011"));
			ad = 1;
			
		}
		
		//Hz/sec = max speed / acc
		var delta = parseFloat(this.getParameterValue("1.006")) / acc;
		
		//convert to 1 tick
		delta = delta /1000 * this.tickInterval;
		
		var newFreq;
		
		newFreq = absFreq + ad * delta;
		
		
		
		if (ad>0) {
			//check we haven't gone too high
			if (newFreq>referenceFreq) {
				newFreq = referenceFreq;
			}
		} else {
			//check we haven't gone too low
			if (newFreq<referenceFreq) {		
				newFreq = referenceFreq;
			}
		}
		
		
		//set the new frequency to the correct direction
		newFreq = newFreq * referenceDir;
		
		this.setParameterValue("5.001", "" + newFreq);
		this.setParameterValue("2.001", "" + newFreq);
		

		//set motor rpm
		var poles = parseInt(this.getParameterValue("5.011"));
		if (poles==0) {
			poles = Math.floor(60 * parseInt(this.getParameterValue("5.006")) / parseInt(this.getParameterValue("5.008")))
		}
		this.setParameterValue("5.004", "" + (newFreq * 60 / poles));
	
		
		this.updateDisplay();
		
	
		
	
	},
	editNum: function(direction) {
		var currMenu = this.menus[this.currentMenu];
		var currParameter = currMenu.parameters[currMenu.currentParameter];
				
		
		//work out the number of dps from current number
		var dp = 0;
		var idot = this.currentParameterValue.indexOf(".");
		if (idot==-1) {
			dp = 0;
		} else {
			dp = this.currentParameterValue.length - idot - 1;
		}
		
		var d = 0;
		if (this.buttonHeldRepeat>0) {
			var d = Math.floor(this.buttonHeldRepeat/10);		//increment digit every 10 repeats when button held down
			if (Math.floor(this.buttonHeldRepeat/10)==this.buttonHeldRepeat/10) this.buttonHeldRepeat++;	//add 1 to sort out the digit rollover
		}

		var inc = Math.pow(10,d-dp);							
		if (inc>10) inc = 10;	//limit the increment
			
		
		var value = parseFloat(this.currentParameterValue);
		
		value += direction * inc;
		
		//validate against max/min values
		if (!isNaN(currParameter['max'])) {
			if (value > parseFloat(currParameter['max'])) {
				value = parseFloat(currParameter['max']);
			}
		}
		
		if (!isNaN(currParameter['min'])) {
			if (value < parseFloat(currParameter['min'])) {
				value = parseFloat(currParameter['min']);
			}
		}
		
		this.currentParameterValue = value.toFixed(dp);
	
	},
	
	displayValue: function(parameterInfo, value) {
		if (parameterInfo.storedParameterName=="5.012") {
			this.showNotImplemented("Autotune not implemented in this demo");
		}
		if (parameterInfo.storedParameterName=="7.055") {
			this.showNotImplemented("Function not implemented in this demo");
		}
		
		switch (parameterInfo.type) {
			case "Num":
				if (parameterInfo.storedParameterName=="11.029") {
					//handle version number formatting
					this.display(value.charAt(1) + value.charAt(0) + "." + value.charAt(3) + value.charAt(2) + "." + value.charAt(5) + value.charAt(4));
				} else {
					this.display(value);
				}
				break;
			case "Txt":
				var dVal = parameterInfo["options"][parseInt(value)];
				this.display(dVal);
				break;
			case "Bit":
				var dVal = parameterInfo["options"][parseInt(value)];
				this.display(dVal);
				break;	
			default:
				//alert("Can't Display " + parameterInfo.type);
				this.showNotImplemented("Parameter not implemented in this demo");
				this.display("-");
		
		}
		
		
	
	},
	
	showNotImplemented: function(text) {
		var ni = d3.select('#not-implemented');
		
		if (ni.empty()) {
			d3.select('body').append('div').attr('id', 'not-implemented');
			ni = d3.select('#not-implemented');

			
		}
		
		ni.html("<span class='fa fa-info-circle'></span> " + text);
		
		ni.style('display', 'block');
	
	},
	
	parameterString: function(menu, parameter) {
		return ("00" + menu).slice(-2) + "." + ("000" + parameter).slice(-3);
	
	},
	
	setupMenus: function() {
		this.menus = [];	
	
		var currMenu = null;
		
		for (var p in this.parameterInfo) {
			var mp = p.split(".");
			var menuName = mp[0];
			var paramName = mp[1];
			
			if ( currMenu == null || currMenu.menuName!= menuName ) {
				currMenu = { menuName: menuName, parameters: [], currentParameter: 0 };
				
				this.menus.push(currMenu);
			}
			
			var parameter = this.parameterInfo[p];
			parameter.parameterName = paramName;
			parameter.storedParameterName = p;
			currMenu.parameters.push(parameter);
			
		}

	},
	
	getParameterInfo: function(pname) {
		var mp = pname.split(".");
		var menuName = ("00" + mp[0]).slice(-2);
		var parameterName = ("000" + mp[1]).slice(-3);
		for (var m in this.menus) {
			if (this.menus[m].menuName==menuName) {
				var menu = this.menus[m];
				for (var p in menu.parameters) {
					if (menu.parameters[p].parameterName==parameterName) {
						return menu.parameters[p];
					}
				}
				
				break;
			}
		
		}
		
		return null;
	
	},
	
	getParameterValue: function(pname) {
		var mp = pname.split(".");
		var menuName = ("00" + mp[0]).slice(-2);
		var parameterName = ("000" + mp[1]).slice(-3);
		
		if (typeof(this.parameterValues[menuName + "." + parameterName])=="undefined") {
			var pi = this.getParameterInfo(pname);
			if (pi!==null && typeof(pi.default)!="undefined") {
				return pi.default;
			}
		}
	
		return this.parameterValues[menuName + "." + parameterName];
	},
	
	setParameterValue: function(pname, value) {
		var mp = pname.split(".");
		var menuName = ("00" + mp[0]).slice(-2);
		var parameterName = ("000" + mp[1]).slice(-3);
	
		var info = this.getParameterInfo(pname);
		if (info && typeof(info.dp)!="undefined") {
			value = parseFloat(value).toFixed(info.dp);
		}
		
		this.parameterValues[menuName + "." + parameterName] = value;
		
		
	},
	
	setupMenu0: function() {
		var menu0 = {menuName:"00", parameters:[], currentParameter:0};
		for (var p=1; p<=95; p++) {
			pname = "22." + ("000"+p).slice(-3);
			
			var storedParameter = this.getParameterValue(pname);
			if (storedParameter && parseInt(storedParameter)!=0) {
				var param = Object.assign({}, this.getParameterInfo(storedParameter));
				
				if (param) {
					param.parameterName = ("000"+p).slice(-3);
					param.storedParameterName = storedParameter;
				
					menu0.parameters.push(param);
				}
			
			}	
		}
		
		this.menus.unshift(menu0);
		
		this.started = true;
	
	},
	setupDisplay: function() {
		var defs = this.svg.select("defs");
		var filter = defs.append("filter").attr("id", "glow")

			
		filter.append("feGaussianBlur")
			.attr("in", "SourceAlpha")
			.attr("stdDeviation", 0.3)
			.attr("result", "blurred");
			
		filter.append("feFlood")
			.attr("flood-color", "rgb(255, 50, 50)")
			.attr("result", "glowColor");
			
		filter.append("feComposite")
			.attr("in", "glowColor")
			.attr("in2", "blurred")
			.attr("operator", "in")
			.attr("result", "glow");	
			
		var feMerge = filter.append("feMerge");
		feMerge.append("feMergeNode").attr("in", "glow")
		feMerge.append("feMergeNode").attr("in", "SourceGraphic");
		
		this.display("");
	},
	
	loadParfile: function(parfile) {
		this.parameterValues = {};
		var that = this;
		return d3.xml(parfile, {credentials: 'include'}).then(function(xml) {
			that.xml = xml;
			var parametersElem = xml.documentElement.getElementsByTagName("parameters")[0];
			var parameters = parametersElem.getElementsByTagName("parameter");
			
			for (var p=0; p<parameters.length; p++) {
				var num = parameters[p].getAttribute("num");
				var value = parameters[p].getAttribute("value");
				
				var mp = num.split(".");
				
				var parameterName = that.parameterString(mp[0], mp[1]);
				
				that.setParameterValue(parameterName, value);
				
			}
			
			that.setParameterValue("5.001", "0");
			that.setParameterValue("2.001", "0");
			that.setParameterValue("5.004", "0");
			
			that.setupMenu0();
			
		});
		
		
		
	},
	
	getParfile: function() {
		var xml = this.xml;
		var parametersElem = xml.documentElement.getElementsByTagName("parameters")[0];
		var parameters = parametersElem.getElementsByTagName("parameter");
	
		//resave the paramaters to the xml
		for (var p=0; p<parameters.length; p++) {
			var num = parameters[p].getAttribute("num");
			
			var mp = num.split(".");
			
			var parameterName = this.parameterString(mp[0], mp[1]);
			
			var value = this.getParameterValue(parameterName);
			
			parameters[p].setAttribute("value", value);
		}
		
		
		xmlString = (new XMLSerializer()).serializeToString(xml);
		
		return xmlString;
	
	},
	
	selectDigit: function(digit) {
		var sel = "";
		for (var s in this.segs) {
			if (sel.length>0) sel += ",";
			
			sel += "path#seg" + digit + this.segs[s];		
		}
		
		return this.svg.selectAll(sel);
	},
	display: function(value) {
		//pad text align to right to right
		var strlen = value.replace(".", "").length;
		for (var pad=0; pad<this.numDigits-strlen; pad++) {
			value = " " + value;
		}
		
		//clear current value
		for (var d=1; d<=this.numDigits; d++) {
			for (var s in this.segs) {
				this.svg.select('path#seg' + d + this.segs[s]).attr("fill-opacity", 0);
			}
			this.svg.select('path#seg' + d + "dot").attr("fill-opacity", 0);
		}
		
		var d=1;
		for (var i=0; i<value.length; i++) {
			var ch = value.charAt(i);
			if (ch in this.segMask && this.segMask[ch]!="") {
				var mask = this.segMask[ch];
				
				for (var m=0; m<mask.length; m++) {
					if (mask.charAt(m)=="1") {
						this.svg.select('path#seg' + d + this.segs[m]).attr("fill-opacity", 1);	
					}
				}
			} else {
				console.log("No mask for char: " + ch);
			} 
			
			
			
			//check if next char is a dot
			if (i<value.length-1 && value.charAt(i+1)==".") {
				this.svg.select('path#seg' + d + "dot").attr("fill-opacity", 1);
				i++;
			} 

			d++;
		
		}
		
	},
	numDigits: 6,
	segs: ["A", "B", "C", "D", "E", "F", "G"],
	segMask: {" ": "0000000", 
				"0":"1111110", 
				"1":"0110000", 
				"2":"1101101", 
				"3":"1111001", 
				"4":"0110011", 
				"5":"1011011", 
				"6":"1011111", 
				"7":"1110000", 
				"8":"1111111", 
				"9":"1111011",
				"A":"1110111",
				"a":"1110111",
				"B":"",
				"b":"",
				"C":"1001110",
				"c":"1001110",
				"D":"0111101",
				"d":"0111101",				
				"E":"1001111",
				"e":"1001111",				
				"F":"1000111",
				"f":"1000111",				
				"G":"",
				"g":"",				
				"H":"0010111",
				"h":"0010111",				
				"I":"0110000",
				"i":"0010000",				
				"J":"",
				"j":"",				
				"K":"",
				"k":"",				
				"L":"0001110",
				"l":"0001110",				
				"M":"",
				"m":"",				
				"N":"1110110",
				"n":"0010101",				
				"O":"1111110",
				"o":"0011101",				
				"P":"1100111",
				"p":"1100111",				
				"Q":"1110011",
				"q":"1110011",				
				"R":"1000110",
				"r":"0000101",				
				"S":"1011011",
				"s":"1011011",				
				"T":"0001111",
				"t":"0001111",				
				"U":"0111110",
				"u":"0011100",				
				"V":"0111110",
				"v":"0011100",				
				"W":"",
				"w":"",				
				"X":"",
				"x":"",				
				"Y":"0111011",
				"y":"0111011",				
				"Z":"",
				"z":"",				
				"-":"0000001"
				}

}
