function PowerTerminals(svg, project) {
	this.dispatch = d3.dispatch("change");
	
	this.svg = svg;
	
	this.project = project;
	this.controlBox = project.controlBox;
	
	this.svg.select("#layer1").on("mousemove", this.mousemoveHandler.bind(this));
	this.svg.select("#layer1").on("click", this.clickHandler.bind(this));
	
	var dt = this;
	
	
	
	
	//set up component terminal
	for (var t=0; t<=this.terminals.length;  t++) {
		var svgt = this.svg.select("#" + this.terminals[t]);
		svgt.on("click", function() {
			d3.event.stopPropagation();
			if (dt.drawingTerminal==false) {				
				dt.drawingTerminal = d3.select(this).attr("id");
				
				//highlight all valid terminals
				/*var validConnections = dt.validConnections[dt.project["powermode"]];
				for (var v = 0; v<validConnections.length; v++) {
					if (validConnections[v][0] == dt.drawingTerminal || validConnections[v][1] == dt.drawingTerminal) {
						dt.svg.select('#' + validConnections[v][0]).style('fill', dt.terminalHighlight);
						dt.svg.select('#' + validConnections[v][1]).style('fill', dt.terminalHighlight);
				    }
				}*/
				
			} else {
				var terminal1 = dt.drawingTerminal;
				var terminal2 = d3.select(this).attr("id");
				
				var validConnections = dt.validConnections[dt.project["powermode"]];
				//check this is a valid connection
				/*var valid = null;
				for (var v = 0; v<validConnections.length; v++) {
					if ( (validConnections[v][0] == terminal1 && validConnections[v][1] == terminal2) ||  (validConnections[v][1] == terminal1 && validConnections[v][0] == terminal2) ) {
						valid = validConnections[v];
						break;
				    }
				}*/
				
				//if (valid!=null) {
					if (typeof(dt.project.powerwiring) == "undefined") dt.project.powerwiring = [];
					
					//filter out any current connections with either terminal
					dt.project.powerwiring = dt.project.powerwiring.filter(function(x) {
									return (x[0]!=terminal1 && x[1]!=terminal1 && x[0]!=terminal2 && x[1]!=terminal2);
								});
					
					
					dt.project.powerwiring.push([terminal1, terminal2]);
					
					dt.clickHandler();
					
					dt.dispatch.call("change");
					
					dt.drawTerminals();
					
				//}
			}
		});
	}

	
	
	
}

PowerTerminals.prototype = {
	terminalColor: "#999999",
	terminalHighlight: "#EA5A00",
	terminals: ["t1_1ph", "t2_1ph", "t1_3ph", "t2_3ph", "t3_3ph", "t1_f1", "t2_f1",  "t1_f2", "t2_f2",  "t1_f3", "t2_f3", "tearth", "e1", "l1", "l2", "l3", "U", "V", "W", "e2", "t_x5F_U", "t_x5F_V", "t_x5F_W", "t_x5F_earth"],
	
	validConnections: { 
			singlephase: [
				["t1_1ph", "t1_f1"],
				["t2_1ph", "t1_f3"],
				["tearth", "e1"],
				["t2_f1", "l3"],
				["t2_f3", "l1"],
				["U", "t_x5F_U"],
				["V", "t_x5F_V"],
				["W", "t_x5F_W"],
				["e2", "t_x5F_earth"]
				
			],
			threephase: [
				["t1_3ph", "t1_f1"],
				["t1_3ph", "t1_f2"],
				["t1_3ph", "t1_f3"],				
				["t2_3ph", "t1_f1"],
				["t2_3ph", "t1_f2"],
				["t2_3ph", "t1_f3"],	
				["t3_3ph", "t1_f1"],
				["t3_3ph", "t1_f2"],
				["t3_3ph", "t1_f3"],	
				["tearth", "e1"],
				["t2_f1", "l1"],
				["t2_f1", "l2"],				
				["t2_f1", "l3"],
				["t2_f2", "l1"],
				["t2_f2", "l2"],				
				["t2_f2", "l3"],
				["t2_f3", "l1"],
				["t2_f3", "l2"],				
				["t2_f3", "l3"],
				["U", "t_x5F_U"],
				["V", "t_x5F_V"],
				["W", "t_x5F_W"],
				["e2", "t_x5F_earth"]
			]
		},
		
	checkComplete: function() {
		//return true if all connections are valid and everything is connected
		//else return array of incorrect terminals
		
		var validConnections = this.validConnections[this.project["powermode"]];
		var powerWiring = this.project.powerwiring;
	
		var errors = [];

		for (var c = 0; c<powerWiring.length; c++) {
			var found = false;
			for (var v=0; v<validConnections.length; v++) {
				if (powerWiring[c][0]==validConnections[v][0] && powerWiring[c][1]==validConnections[v][1]) {
					found=true;
				}
			}
			if (!found) {
				errors.push(powerWiring[c][0]);
				errors.push(powerWiring[c][1]);
			}
		}
		
		//check everything in the validconnections is at least connected to something
		for (var v=0; v<validConnections.length; v++) {
			var ok0 = false;
			var ok1 = false;
			for (var c = 0; c<powerWiring.length; c++) {
				if (validConnections[v][0]==powerWiring[c][0] || validConnections[v][0]==powerWiring[c][1]) {
					ok0=true;
				}
				if (validConnections[v][1]==powerWiring[c][0] || validConnections[v][1]==powerWiring[c][1]) {
					ok1=true;
				}
			}
			
			if (!ok0) {
				errors.push(validConnections[v][0]);
			}
			if (!ok1) {
				errors.push(validConnections[v][1]);
			}
			
		
		}
		
		
		if (errors.length==0) {
			return true;
		} else {
			return errors;
		}
	},	
	
	line1: ["tearth", "t_x5F_W", "t_x5F_V", "t_x5F_U", "t_x5F_earth"],
	line2: [ "W", "V", "U"],
	line3: [ "t1_1ph", "t2_1ph", "t1_3ph", "t2_3ph", "t3_3ph"],
	linez: ["t2_f1", "t2_f2", "t2_f3", "l1", "l2", "l3", "e2"],
	
	
	drawTerminals: function() {
		var dt = this;
		
		
		//remove any connections with non-existant terminals (esp if changing from 1ph to 3ph)
		if (typeof(this.project.powerwiring) == "undefined") this.project.powerwiring = [];
		dt.project.powerwiring = dt.project.powerwiring.filter(function(x) {
				var validConnections = dt.validConnections[dt.project["powermode"]];
				var valid0=false;
				var valid1=false;
				for (var v = 0; v<validConnections.length; v++) {
					if (x[0]==validConnections[v][0] || x[0]== validConnections[v][1]) {
						valid0=true;
					}
					if (x[1]==validConnections[v][0] || x[1]== validConnections[v][1]) {
						valid1=true;
					}
				}
				return valid0 && valid1;
			});
						
		
		
		//clear all current
		this.svg.selectAll("use").remove();
		this.svg.selectAll(".componentLine").remove();
		
		for (var t=0; t<=this.terminals.length;  t++) {
			this.svg.select("#" + this.terminals[t]).style("fill", this.terminalColor);
		}
		
		//show hide appropriate power terminals
		if (this.project["powermode"]=="singlephase") {
			d3.select("#singlephase").attr("visibility", "visible");
			d3.select("#threephase").attr("visibility", "hidden");
			d3.select("#f2").attr("visibility", "hidden");
		} else {
			d3.select("#singlephase").attr("visibility", "hidden");
			d3.select("#threephase").attr("visibility", "visible");
			d3.select("#f2").attr("visibility", "visible");
		}
		
		
		if (typeof(this.project.powerwiring) != "undefined") {
			for (var w = 0; w<this.project.powerwiring.length; w++) {
				var t1 = this.project.powerwiring[w][0];
				var t2 = this.project.powerwiring[w][1];
				
				var terminal1 = this.svg.select("#" + t1);
				var terminal2 = this.svg.select("#" + t2);
				
				
				var xy1 = [terminal1.attr("cx"), terminal1.attr("cy")];
				var xy2 = [terminal2.attr("cx"), terminal2.attr("cy")];
					 	
		   	 	var path = this.svg.select('#layer1')
		             .append('path')
		             .style('stroke', "#ee7930")
		             .style('stroke-width', '2')
		             .style('stroke-linecap', 'round')
		             .style('stroke-linejoin', 'round')
		             .style('fill', 'none')
		             .attr('class', "componentLine")
		             .attr('d', this.lineFunction(xy1, xy2, t1));
			
			
			}
		
		}
	
	},
	
	drawingTerminal: false,
	drawingPath: null,
	mousemoveHandler: function() {
		if (this.drawingTerminal!==false) {
			var svgt = this.svg.select("#" + this.drawingTerminal);
			
			var xy0 = [svgt.attr("cx"), svgt.attr("cy")];
			
			var xy1 = d3.mouse(this.svg.select("#layer1").node()).map(function(x){ return x - 2; });
			
			var line = this.lineFunction(xy0, xy1, this.drawingTerminal)
			
			if (this.drawingPath==null) {
				//create the path
				
				this.drawingPath = this.svg.select('#layer1')
                     .append('path')
                     .style('stroke', this.terminalHighlight)
                     .style('stroke-width', '1px')
                     .style('fill', 'none')
                     .attr('d', line);
			} else {
				//move the path
				this.drawingPath.attr("d", line);
			}
		
		}
	
	},
	
	clickHandler: function() {
		if (this.drawingTerminal!==false) {
			this.drawingTerminal = false;
			this.drawingPath.remove();
			this.drawingPath = null;
			
			for (var t=0; t<=this.terminals.length;  t++) {
				this.svg.select("#" + this.terminals[t]).style("fill", this.terminalColor);
			}
		}
	},
	
	
		
	lineFunction: function(start, end, terminal1) {
		
		if (this.line1.indexOf(terminal1)!=-1) {
			var midx = (parseFloat(start[0]) + parseFloat(end[0])) / 2;
			
			var xy1 = [midx, start[1]];
			var xy2 = [midx, end[1]];
			
			return d3.line()
				.x(function(d){ return d[0]; })
				.y(function(d){ return d[1]; })([start, xy1, xy2, end]);
		
		} else if (this.line2.indexOf(terminal1)!=-1) {
			var ix = this.line2.indexOf(terminal1);
			
			var midx = (parseFloat(start[0]) + parseFloat(end[0])) / 2 + (50*ix) ;
			
			var ylow = parseFloat(start[1]) + (20*(ix+1));
			var yhi = parseFloat(end[1]) - (20*(3-ix));
		
			
			return d3.line()
				.x(function(d){ return d[0]; })
				.y(function(d){ return d[1]; })([start, [start[0], ylow], [midx, ylow], [midx, yhi], [end[0], yhi], end]);
		
		} else if (this.line3.indexOf(terminal1)!=-1) {
			
			var xy1 = [start[0], end[1]];
			
			return d3.line()
				.x(function(d){ return d[0]; })
				.y(function(d){ return d[1]; })([start, xy1, end]);
		
		} else {
			
			var xy1 = [end[0], start[1]];
			
			return d3.line()
				.x(function(d){ return d[0]; })
				.y(function(d){ return d[1]; })([start, xy1, end]);
		
		}
		
	}
}
