function DriveTerminals(svg, project) {
	this.dispatch = d3.dispatch("change");
	
	this.svg = svg;
	
	this.project = project;
	this.controlBox = project.controlBox;
	
	this.svg.select("#layer1").on("mousemove", this.mousemoveHandler.bind(this));
	this.svg.select("#layer1").on("click", this.clickHandler.bind(this));
	
	var dt = this;
	
	
	//create tooltip 
	var tooltip = d3.select("body").append("div")	
		.attr("class", "tooltip")				
		.style("opacity", 0);
	
	//set up component terminal events
	for (var c=0; c<10; c++) {
		for (var t=1; t<=3;  t++) {
			var svgt = this.svg.select("#t" + (c+1) + t);
			svgt.attr("c", c);
			svgt.attr("t", t);
			svgt.on("click", function() {
				d3.event.stopPropagation();
				dt.drawingComponent = d3.select(this).attr("c");
				dt.drawingTerminal = d3.select(this).attr("t");			
				
				dt.mousemoveHandler();
			});
		}
		
		
		//wire up delete events
		this.svg.select("#close_x5F_control" + (c+1)).attr("c", c);
		this.svg.select("#close_x5F_control" + (c+1)).on("click", function() {
			d3.event.stopPropagation();
			dt.project.controlBox[d3.select(this).attr("c")] = null;
			dt.drawTerminals();
			
			dt.dispatch.call("change");
		});
		
		
		

		this.svg.select("#info" + (c+1)).attr("c", c);
		this.svg.select("#info" + (c+1)).on("mouseover", function() {
			
			var ctl =  dt.project.controlBox[d3.select(this).attr("c")];
			
			if (ctl && ctl['name']) {
				tooltip.transition().duration(200).style("opacity", .9);
				tooltip.html(ctl['name'])
					.style("left",  (d3.event.pageX) + "px")		
                	.style("top", (d3.event.pageY - 28) + "px");	
                
			}
			
		}).on("mouseout", function(d) {		
            tooltip.transition()		
                .duration(500)		
                .style("opacity", 0);	
        });
		

	}
	
	
	//set up drive terminal events
	for (var t in this.driveTerminals) {
		this.svg.select("#" + this.driveTerminals[t].name).on("click", function() {
			d3.event.stopPropagation();
			if (dt.drawingTerminal!==false) {
				

				dt.controlBox[dt.drawingComponent]["terminal"+ dt.drawingTerminal] = d3.select(this).attr("id");				
				
				dt.clickHandler();
				dt.drawTerminals();
				
				dt.dispatch.call("change");
			}		
		
		});
	}
	
				
	
}

DriveTerminals.prototype = {
	terminalColor: "#999999",
	terminalHighlight: "#EA5A00",
	pr05: [ "AV", "AI", "AV.Pr", "AI.Pr", "Preset", "Pad", "Pad.ref", "E.Pot", "torque", "Pid"],
	
	drawTerminals: function() {
		var dt = this;
		
		//clear all current
		this.svg.selectAll("use").remove();
		this.svg.selectAll(".componentLine").remove();
		
		//reset colours
		for (var t in this.driveTerminals) {
			this.svg.select("#" + this.driveTerminals[t].name).style('fill', this.terminalColor);
		}
		
		for (var c=0; c<this.controlBox.length; c++) {
			if (this.controlBox[c]!=null) {
				var component = this.controlBox[c];
				var componentType = component.component;
				var componentInfo = this.components[componentType];
				
				//show the delete button
				this.svg.select("#close_x5F_control" + (c+1)).classed("terminalhidden", false);
				
				//show the info button
				this.svg.select("#info" + (c+1)).classed("terminalhidden", false);
				
				
				
				var uc = this.svg.select("#usecomponent" + c);
				
				if (uc.empty()) {
					var container = this.svg.select('#control' + (c+1));
					
					uc = this.svg.select("#layer1").append("use").attr("id", "usecomponent" + c).attr("xlink:href","#component_" + componentType)

					var containerbox = container.node().getBBox();
					
					uc.attr("x", containerbox.x);
					uc.attr("y", containerbox.y);
					uc.attr("width", containerbox.width);
					uc.attr("height", containerbox.height);
					
					if (c >=5 ) {
						var tx = parseFloat(containerbox.x) * 2 + parseFloat(containerbox.width);
						uc.attr("transform", "translate(" + tx + ", 0) scale(-1, 1)");
					}
					
				}
				
				for (var t=1; t<=3; t++) {
					var svgt = this.svg.select("#t" + (c+1) + t);
					
					
					//set terminal visibility
					if (componentInfo['terminal'+t]) {
						svgt.classed("terminalhidden", false);
					
					} else {
						svgt.classed("terminalhidden", true);
					
					}
					
					
					if (component['terminal'+t]) {
						var componentObject = svgt;
						
						var driveTermInfo = null;
						var dtx;
						for (dtx = 0; dtx<this.driveTerminals.length; dtx++) {
							if (this.driveTerminals[dtx].name==component['terminal'+t]) {
								driveTermInfo = this.driveTerminals[dtx];
								break;
							}
						}
						
						
						var driveObject = this.svg.select("#" + component['terminal'+t]);
					
						//wire up terminals
						
					 	var xy0 = [parseInt(componentObject.attr("cx")) + ((c>4) ? -5 : 5), componentObject.attr("cy")];
					 	
					 	var terminalx = parseFloat(driveObject.attr("cx"));
					 	var terminaly = parseFloat(driveObject.attr("cy"));
					 	var xy1 = [terminalx, terminaly];
				   	 	var path = this.svg.select('#layer1')
				             .append('path')
				             .style('stroke', driveTermInfo.color)
				             .style('stroke-width', '2')
				             .style('fill', 'none')
				             .attr('class', "componentLine")
				             .attr('d', this.lineFunction(xy0, xy1, dtx, c, t));
				   }
					
				}
	
			
			} else {
				//hide the terminal connectors
				for (var t=1; t<=3; t++) {
					this.svg.select("#t" + (c+1) + t).classed("terminalhidden", true);
				
				}
				
				//hide the delete button
				this.svg.select("#close_x5F_control" + (c+1)).classed("terminalhidden", true);
				//hide the info button
				this.svg.select("#info" + (c+1)).classed("terminalhidden", true);
				
				
				var requiredComponents = dt.requiredComponents();
				var canAdd = false;
				for (var r in requiredComponents) {
					if (requiredComponents[r]>0) {
						canAdd = true;
						break;
					}
				}
				
				var ac = this.svg.select("#btnAddComponent");
				if (canAdd && ac.empty()) {
					var container = this.svg.select('#control' + (c+1));
					
					ac = this.svg.select("#layer1").append("use").attr("id", "btnAddComponent").attr("xlink:href","#addComponent")

					var containerbox = container.node().getBBox();
					
					ac.attr("x", containerbox.x);
					ac.attr("y", containerbox.y);
					ac.attr("width", "100%");
					ac.attr("height", "100%");
					
					ac.on("click", function() {
					
						var requiredComponents = dt.requiredComponents();
						
						d3.selectAll('#selectComponentDialog .selectComponent').each(function() {
							var componentType = d3.select(this).select("a").attr("data-component");
							if (requiredComponents[componentType]>0) {
								d3.select(this).style('display', '');
							} else {
								d3.select(this).style('display', 'none');
							}
						});
						
						d3.select('#selectComponentDialog .step2').style("display", "none");
						
						
						$('#selectComponentDialog .strategy').text(dt.pr05[dt['project']['drivemode']]);
						$('#selectComponentDialog').show().dialog({
							modal		: true,
							fluid		: true,
							resizeable	: false,
							width		: 'auto',
							height		: 'auto',
							maxWidth	: 1160,
							title		: $('#selectComponentDialog h1').text(),
							close		: function(){ },
							beforeClose	: function(){
								return true;
							}
						});
					});
					
					var addComponentSlot = c;
					
					var selectedComponent = "";
					
					d3.selectAll("#selectComponentDialog a.select").on("click", function() {
						d3.event.preventDefault();

						selectedComponent = d3.select(this).attr("data-component");
						
						var selectBox = d3.select("#selectComponentDialog select#selectName");
						selectBox.selectAll("option").remove();
						
						var validConfig = DriveConfiguration.validConfigs[dt.project['drivemode']];
						
						for (var v=0; v<validConfig.length; v++) {
							if (validConfig[v].component==selectedComponent) {
								var used=false;
								for (var cc=0; cc<dt.controlBox.length; cc++) {
									if (dt.controlBox[cc] != null && validConfig[v]['name']==dt.controlBox[cc]['name']) {
										used=true;
										break;
									}
								
								}
								
								if (!used) {
									selectBox.append("option").text(validConfig[v].name);
								}
							}
						
						}
						
						d3.select('#selectComponentDialog .step2').style("display", "block");
						
					});
					
					
					d3.select("#selectComponentDialog a.ok").on("click", function() {
						
						dt.controlBox[addComponentSlot] = {name: d3.select("#selectComponentDialog select#selectName").property("value"), component: selectedComponent, terminal1: null, terminal2: null, terminal3: null }
						dt.drawTerminals();
						$('#selectComponentDialog').dialog("close");
						
						dt.dispatch.call("change");
					
					});
				}
				
			}
		}
	
	},
	
	drawingComponent: false,
	drawingTerminal: false,
	drawingPath: null,
	mousemoveHandler: function() {
		if (this.drawingTerminal!==false) {
			var foo = "#t" + (parseInt(this.drawingComponent)+1) + this.drawingTerminal;
			var svgt = this.svg.select(foo);
			var xy0 = [svgt.attr("cx"), svgt.attr("cy")];
			var xy1 = d3.mouse(this.svg.select("#layer1").node()).map(function(x){ return x - 2; });
			
			var line = this.lineFunction(xy0, xy1, 0, this.drawingComponent, this.drawingTerminal)
			
			if (this.drawingPath==null) {
				//create the path
				
				this.drawingPath = this.svg.select('#layer1')
                     .append('path')
                     .style('stroke', this.terminalHighlight)
                     .style('stroke-width', '2')
                     .style('fill', 'none')
                     .attr('d', line);
			} else {
				//move the path
				this.drawingPath.attr("d", line);
			}
		
		}
	
	},
	
	clickHandler: function() {
		if (this.drawingTerminal!==false) {
			this.drawingComponent = false;
			this.drawingTerminal = false;
			this.drawingPath.remove();
			this.drawingPath = null;
			
			for (var t in this.driveTerminals) {
				this.svg.select("#" + this.driveTerminals[t].name).style('fill', this.terminalColor);
			}
		}
	},
	
	
	
	components: {
			"led": {
				terminal1: true,
				terminal2: false,
				terminal3: true
			}, 
			"switch": {
				terminal1: true,
				terminal2: false,
				terminal3: true
			}, 
			"potentiometer": {
				terminal1: true,
				terminal2: true,
				terminal3: true
			},
			"analogmeter": {
				terminal1: true,
				terminal2: false,
				terminal3: true
			},
			"pushbutton": {
				terminal1: true,
				terminal2: false,
				terminal3: true
			}, 
			"currentsource": {
				terminal1: true,
				terminal2: false,
				terminal3: true
			}
		},
		
	driveTerminals: [
			{ name: "terminal1", type: "GND", color: "black" },
			{ name: "terminal2", type: "ANALOGIN", color: "green" },
			{ name: "terminal4", type: "10V", color: "red" },
			{ name: "terminal5", type: "ANALOGIN", color: "green" },
			{ name: "terminal7", type: "ANALOGOUT", color: "darkgreen" },
			{ name: "terminal9", type: "24V", color: "darkred" },
			{ name: "terminal10", type: "DIO", color: "blue" },
			{ name: "terminal11", type: "DIN", color: "darkblue" },
			{ name: "terminal12", type: "DIN", color: "darkblue" },
			{ name: "terminal13", type: "DIN", color: "darkblue" },
			{ name: "terminal14", type: "DIN", color: "darkblue" }
			
		],
		
	
	checkComplete: function() {
		var validConfig = DriveConfiguration.validConfigs[this.project['drivemode']];
		
		var errors = [];
		for (var c=0; c<this.controlBox.length; c++) {
			var control = this.controlBox[c];
			if (control!==null) {
				var vc = null;
				for (var v=0; v<validConfig.length; v++) {
					if (validConfig[v].name==control.name) {
						vc = validConfig[v];
						break;	
					}
				
				}		
				
				if (vc) {
					for (var t=1; t<=3; t++) {
					
						if (control["terminal" + t]!=vc["terminal" + t]) {
							//terminals don't match - check if terminals can be switched
							if (typeof(vc.allowSwap)!="undefined") {
								var swaps = vc.allowSwap.match(/.{1,2}/g);
								for (var s=0; s<swaps.length; s++) {
									var si = swaps[s].indexOf(t.toString());
									if (si!=-1) {
										//find the other character
										var swapwith = swaps[s].charAt((si==0? 1 : 0));
										if (control['terminal' + t] != vc['terminal' + swapwith]) {
											errors.push("t" + (c+1) + t);
										}
									}
								}
							
							}
						
						} //else no error
				
					}
				} else {
					//control not found, highlight all terminals
					errors.push("t" + (c+1) + "1");
					errors.push("t" + (c+1) + "2");
					errors.push("t" + (c+1) + "3");								
				
				}
			}
		}
		
		if (errors.length==0) {
			return true;
		} else {
			return errors;
		}
		
	},
	
	requiredComponents: function() {
		//return array of component names that are still required
		var validConfig = DriveConfiguration.validConfigs[this.project['drivemode']];
		
		if (typeof(validConfig)=="undefined") return {};		//new project - no drivemode set
		
		var numComponents = {};
		
		//find number of each component type		
		for (var v = 0; v<validConfig.length; v++) {
			var componentType = validConfig[v]['component'];
			if (typeof (numComponents[componentType])=="undefined" ) {
				numComponents[componentType] = 1;
			} else {
				numComponents[componentType]++;
			}
		}
		
		//go through the control box and reduce numcomponents required
		for (var c=0; c<this.controlBox.length; c++) {
			if (this.controlBox[c]!=null) {
				numComponents[this.controlBox[c].component] --;
			}
		}
	
		return numComponents;
	},
	
	
		
	lineFunction: function(start, end, dtx, control, terminal) {
		
		
		if (control<5) {
			var xmid = parseFloat(start[0]) + (control*30 + terminal*10) + 12;
		} else {
			var xmid = parseFloat(start[0]) - ((control-5)*30 + terminal*10) - 12;
		}
		
		var ymid = 500 + (dtx * 10);
		
		if (control<3 || (control>=5 && control<8)) {
			var xy1 = [xmid, start[1]];
			var xy2 = [xmid, ymid];
			var xy3 = [end[0], ymid];
			
			return d3.line()
				.x(function(d){ return d[0]; })
				.y(function(d){ return d[1]; })([start, xy1, xy2, xy3, end]);
		} else {
			var xy1 = [end[0], start[1]];
			
			return d3.line()
				.x(function(d){ return d[0]; })
				.y(function(d){ return d[1]; })([start, xy1, end]);
		
		}

		
	}
}
